package spelers.eersteOrde;

import java.util.ArrayList;
import java.util.Iterator;

import simulatie.Model;
import spelers.abstracts.AbstractBurger;
import spelers.abstracts.AbstractWeerwolf;

public class EenWeerwolf extends AbstractWeerwolf{

	public EenWeerwolf(Model controller, String naam, int id) {
		super(controller, naam, id);
		this.orde = eersteOrde;
	}
	
	
	@Override
	//Probeer uit te zoeken welke burgers mogelijkerwijs ziener/onschuldig meisje zijn
	//Doe dit aan de hand van het stemgedrag: als een burger op een andere burger stemt,
	//is die burger in ieder geval geen onschuldig meisje (aangezien OM altijd correct op een wolf stemt,
	//Als een burger op een weerwolf stemt is het een risicogeval dus verdient bij stemmen prioriteit
	public void updateAndereSpelerInschattingen() {
		Iterator<AbstractBurger> it = model.burgers.iterator();
		while (it.hasNext()) {
			AbstractBurger itBurger = (AbstractBurger) it.next();
			if (itBurger.inLeven == true){
				int burgersStem = model.laatsteBrandstapelStemgedrag[itBurger.id];
				//Als een burger correct op een weerwolf heeft gestemd, is het mogelijk meer dan een gewone
				//burger. Als eerder al was uitgesloten dat het een OM is, is het alleen nog mogelijk
				//een ziener.
				if (andereSpelerInschattingen.get(burgersStem).equals("Weerwolf")){
					//Nog mogelijk dat het een OM is
					if (andereSpelerInschattingen.get(itBurger.id).equals(burgerOfZienerOfOM) | 
							andereSpelerInschattingen.get(itBurger.id).equals(verhoogdRisicoOpZienerOfOM)){
						andereSpelerInschattingen.set(itBurger.id, verhoogdRisicoOpZienerOfOM);
					}
					//Dan is eerder uitgesloten dat het een OM is, dus kan het nu alleen nog maar mogelijk een ziener zijn.
					else{
						andereSpelerInschattingen.set(itBurger.id, verhoogdRisicoOpZiener);
					}
				}
				
				else{
					//Als een burger ooit op een andere burger heeft gestemd, is het in ieder geval geen OM
					andereSpelerInschattingen.set(itBurger.id, burgerOfZiener);
				}
			}
		}
		
	}
	
	
	@Override
	//Zoek eerst op mogelijke OM, dan op mogelijk zieners,
	//dan op burgers
	public int opEetBurgerID() {		
		Iterator<AbstractBurger> it = model.burgers.iterator();
		ArrayList<Integer> kandidaten = new ArrayList<Integer>();
		//Check eerst op "Mogelijk onschuldig meisje/ziener", die hebben prioriteit
		while (it.hasNext()) {
			AbstractBurger itBurger = (AbstractBurger) it.next();
			if (itBurger.inLeven == true & 
					andereSpelerInschattingen.get(itBurger.id).equals(verhoogdRisicoOpZienerOfOM)){
				kandidaten.add(itBurger.id);
			}
		}
		
		if (kandidaten.size()>0){
			int keuzeOpeetStemIndex = r.nextInt(kandidaten.size());
			int keuzeOpeetStem = kandidaten.get(keuzeOpeetStemIndex);
			
			System.out.println(this.rol +" "+ this.naam + " vermoedt op basis van zijn/haar stemgedrag dat " + 
					model.spelers.get(keuzeOpeetStem).naam + " een ziener of zelfs onschuldig meisje is, "+ 
					"dus wil daarom deze persoon opeten.");
			return keuzeOpeetStem;
		}
		
		//Bij geen vermoedelijk onschuldig meisje, eet een vermoedelijke ziener.
		kandidaten.clear();
		
		it = model.burgers.iterator();
		
		while (it.hasNext()) {
			AbstractBurger itBurger = (AbstractBurger) it.next();
			if (itBurger.inLeven == true & 
					andereSpelerInschattingen.get(itBurger.id).equals(verhoogdRisicoOpZiener)){
				kandidaten.add(itBurger.id);
			}
		}
		
		if (kandidaten.size()>0){
			int keuzeOpeetStemIndex = r.nextInt(kandidaten.size());
			int keuzeOpeetStem = kandidaten.get(keuzeOpeetStemIndex);
			
			System.out.println(this.rol +" "+ this.naam + " vermoedt op basis van zijn/haar stemgedrag dat " + 
					model.spelers.get(keuzeOpeetStem).naam + " een ziener is, "+ 
					"dus wil daarom deze persoon opeten.");
			return keuzeOpeetStem;
		}
		
		//Bij geen vermoedelijk onschuldig meisje of ziener, eet een willekeurige burger.
		kandidaten.clear();
		
		for (int s =0; s < model.geefAantalSpelers(); s++){
    		if (andereSpelerInschattingen.get(s).equals(burgerOfZienerOfOM) | andereSpelerInschattingen.get(s).equals(burgerOfZiener)
    				& model.spelers.get(s).inLeven == true){
    			kandidaten.add(model.spelers.get(s).id);
    		}
    	}
		int keuzeOpeetStemIndex = r.nextInt(kandidaten.size());
		int keuzeOpeetStem = kandidaten.get(keuzeOpeetStemIndex);

		System.out.println(this.rol +" "+ this.naam + " heeft (nog) geen kennis over wie mogelijk een onschuldig "
				+ "meisje/ziener is, dus wil daarom de willekeurige persoon " + model.spelers.get(keuzeOpeetStem).naam + 
				" opeten.");

		return keuzeOpeetStem;
	}

	
	
	@Override
	//Weerwolven stemmen op een burger om te verbranden	
	public int stemVerbrandenID() {		
		Iterator<AbstractBurger> it = model.burgers.iterator();
		ArrayList<Integer> kandidaten = new ArrayList<Integer>();
		//Check eerst op "Mogelijk onschuldig meisje/ziener", die hebben prioriteit
		while (it.hasNext()) {
			AbstractBurger itBurger = (AbstractBurger) it.next();
			if (itBurger.inLeven == true & 
					andereSpelerInschattingen.get(itBurger.id).equals(verhoogdRisicoOpZienerOfOM)){
				kandidaten.add(itBurger.id);
			}
		}
		
		if (kandidaten.size()>0){
			int keuzeOpeetStemIndex = r.nextInt(kandidaten.size());
			int keuzeOpeetStem = kandidaten.get(keuzeOpeetStemIndex);
			
			System.out.println(this.rol +" "+ this.naam + " vermoedt op basis van zijn/haar stemgedrag dat " + 
					model.spelers.get(keuzeOpeetStem).naam + " een ziener of zelfs onschuldig meisje is, "+ 
					"dus wil daarom deze persoon verbranden.");
			return keuzeOpeetStem;
		}
		
		//Bij geen vermoedelijk onschuldig meisje, eet een vermoedelijke ziener.
		kandidaten.clear();
		
		it = model.burgers.iterator();
		
		while (it.hasNext()) {
			AbstractBurger itBurger = (AbstractBurger) it.next();
			if (itBurger.inLeven == true & 
					andereSpelerInschattingen.get(itBurger.id).equals(verhoogdRisicoOpZiener)){
				kandidaten.add(itBurger.id);
			}
		}
		
		if (kandidaten.size()>0){
			int keuzeOpeetStemIndex = r.nextInt(kandidaten.size());
			int keuzeOpeetStem = kandidaten.get(keuzeOpeetStemIndex);
			
			System.out.println(this.rol +" "+ this.naam + " vermoedt op basis van zijn/haar stemgedrag dat " + 
					model.spelers.get(keuzeOpeetStem).naam + " een ziener is, "+ 
					"dus wil daarom deze persoon verbranden.");
			return keuzeOpeetStem;
		}
		
		//Bij geen vermoedelijk onschuldig meisje of ziener, eet een willekeurige burger.
		kandidaten.clear();
		
		for (int s =0; s < model.geefAantalSpelers(); s++){
    		if (andereSpelerInschattingen.get(s).equals(burgerOfZienerOfOM) | andereSpelerInschattingen.get(s).equals(burgerOfZiener)
    				& model.spelers.get(s).inLeven == true){
    			kandidaten.add(model.spelers.get(s).id);
    		}
    	}
		int keuzeOpeetStemIndex = r.nextInt(kandidaten.size());
		int keuzeOpeetStem = kandidaten.get(keuzeOpeetStemIndex);

		System.out.println(this.rol +" "+ this.naam + " heeft (nog) geen kennis over wie mogelijk een onschuldig"
				+ " meisje/ziener is, dus wil daarom de willekeurige burger " + model.spelers.get(keuzeOpeetStem).naam + 
				" verbranden.");

		return keuzeOpeetStem;
	}
}
